@@ -21,6 +21,7 @@ from collections import deque
 from warnings import warn
 
 from gevent import sleep, spawn, get_hub
+from gevent.lock import RLock
 from ssh2.error_codes import LIBSSH2_ERROR_EAGAIN
 from ssh2.exceptions import SFTPHandleError, SFTPProtocolError, \
     Timeout as SSH2Timeout
@@ -44,6 +45,8 @@ THREAD_POOL = get_hub().threadpool
 
 class SSHClient(BaseSSHClient):
     """ssh2-python (libssh2) based non-blocking SSH client."""
+    # 2MB buffer
+    _BUF_SIZE = 2048 * 1024
 
     def __init__(self, host,
                  user=None, password=None, port=None,
@@ -58,7 +61,9 @@ class SSHClient(BaseSSHClient):
                  proxy_user=None,
                  proxy_password=None,
                  _auth_thread_pool=True, keepalive_seconds=60,
-                 identity_auth=True,):
+                 identity_auth=True,
+                 ipv6_only=False,
+                 ):
         """:param host: Host name or IP to connect to.
         :type host: str
         :param user: User to connect as. Defaults to logged in user.
@@ -95,6 +100,11 @@ class SSHClient(BaseSSHClient):
         :type proxy_port: int
         :param keepalive_seconds: Interval of keep alive messages being sent to
           server. Set to ``0`` or ``False`` to disable.
+        :type keepalive_seconds: int
+        :param ipv6_only: Choose IPv6 addresses only if multiple are available
+          for the host or raise NoIPv6AddressFoundError otherwise. Note this will
+          disable connecting to an IPv4 address if an IP address is provided instead.
+        :type ipv6_only: bool
 
         :raises: :py:class:`pssh.exceptions.PKeyFileError` on errors finding
           provided private key.
@@ -120,13 +130,16 @@ class SSHClient(BaseSSHClient):
                 identity_auth=identity_auth,
             )
             proxy_host = '127.0.0.1'
+        self._chan_lock = RLock()
         super(SSHClient, self).__init__(
             host, user=user, password=password, port=port, pkey=pkey,
             num_retries=num_retries, retry_delay=retry_delay,
             allow_agent=allow_agent, _auth_thread_pool=_auth_thread_pool,
             timeout=timeout,
             proxy_host=proxy_host, proxy_port=proxy_port,
-            identity_auth=identity_auth)
+            identity_auth=identity_auth,
+            ipv6_only=ipv6_only,
+        )
 
     def _shell(self, channel):
         return self._eagain(channel.shell)
@@ -216,8 +229,6 @@ class SSHClient(BaseSSHClient):
             logger.error(msg, self.host, self.port, ex)
             if isinstance(ex, SSH2Timeout):
                 raise Timeout(msg, self.host, self.port, ex)
-            ex.host = self.host
-            ex.port = self.port
             raise
 
     def _keepalive(self):
@@ -284,10 +295,12 @@ class SSHClient(BaseSSHClient):
     def _read_output_to_buffer(self, read_func, _buffer):
         try:
             while True:
-                size, data = read_func()
+                with self._chan_lock:
+                    size, data = read_func()
                 while size == LIBSSH2_ERROR_EAGAIN:
                     self.poll()
-                    size, data = read_func()
+                    with self._chan_lock:
+                        size, data = read_func()
                 if size <= 0:
                     break
                 _buffer.write(data)
@@ -318,8 +331,9 @@ class SSHClient(BaseSSHClient):
         self.close_channel(channel)
 
     def close_channel(self, channel):
-        logger.debug("Closing channel")
-        self._eagain(channel.close)
+        with self._chan_lock:
+            logger.debug("Closing channel")
+            self._eagain(channel.close)
 
     def _eagain(self, func, *args, **kwargs):
         return self._eagain_errcode(func, LIBSSH2_ERROR_EAGAIN, *args, **kwargs)
@@ -399,9 +413,11 @@ class SSHClient(BaseSSHClient):
                     local_file, self.host, remote_file)
 
     def _sftp_put(self, remote_fh, local_file):
-        with open(local_file, 'rb', 2097152) as local_fh:
-            for data in local_fh:
+        with open(local_file, 'rb', self._BUF_SIZE) as local_fh:
+            data = local_fh.read(self._BUF_SIZE)
+            while data:
                 self.eagain_write(remote_fh.write, data)
+                data = local_fh.read(self._BUF_SIZE)
 
     def sftp_put(self, sftp, local_file, remote_file):
         mode = LIBSSH2_SFTP_S_IRUSR | \
@@ -474,9 +490,9 @@ class SSHClient(BaseSSHClient):
         try:
             self._eagain(sftp.stat, remote_file)
         except (SFTPHandleError, SFTPProtocolError):
-            msg = "Remote file or directory %s does not exist"
-            logger.error(msg, remote_file)
-            raise SFTPIOError(msg, remote_file)
+            msg = "Remote file or directory %s on host %s does not exist"
+            logger.error(msg, remote_file, self.host)
+            raise SFTPIOError(msg, remote_file, self.host)
         try:
             dir_h = self._sftp_openfh(sftp.opendir, remote_file)
         except SFTPError:
